@@ -54,7 +54,7 @@ type ShapeList<'model> = Vec<(&'model ShapeID, &'model AppliedTraits, &'model Sh
 // This enum may be extended in the future if other variations are required.
 // It's recursively composable, so you could represent &Option<&Value>
 // with `Ty::Ref(Ty::Opt(Ty::Ref(id)))`
-enum Ty<'typ> {
+pub(crate) enum Ty<'typ> {
     /// write a plain shape declaration
     Shape(&'typ ShapeID),
     /// write a type wrapped in Option<>
@@ -66,10 +66,10 @@ enum Ty<'typ> {
 #[derive(Default)]
 pub struct RustCodeGen<'model> {
     /// if set, limits declaration output to this namespace only
-    namespace: Option<NamespaceID>,
-    packages: HashMap<String, PackageName>,
-    import_core: String,
-    model: Option<&'model Model>,
+    pub(crate) namespace: Option<NamespaceID>,
+    pub(crate) packages: HashMap<String, PackageName>,
+    pub(crate) import_core: String,
+    pub(crate) model: Option<&'model Model>,
 }
 
 impl<'model> RustCodeGen<'model> {
@@ -83,6 +83,7 @@ impl<'model> RustCodeGen<'model> {
     }
 }
 
+#[non_exhaustive]
 enum MethodArgFlags {
     Normal,
     // arg is type ToString
@@ -116,7 +117,13 @@ impl<'model> CodeGen for RustCodeGen<'model> {
     }
 
     fn source_formatter(&self) -> Result<Box<dyn SourceFormatter>> {
-        Ok(Box::new(crate::format::RustSourceFormatter::default()))
+        cfg_if::cfg_if! {
+            if #[cfg(not(target_arch = "wasm32"))] {
+                Ok(Box::new(crate::format::RustSourceFormatter::default()))
+            } else {
+                Ok(Box::new(crate::format::NullFormatter::default()))
+            }
+        }
     }
 
     /// Perform any initialization required prior to code generation for a file
@@ -160,8 +167,15 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         model: &Model,
         _params: &ParamMap,
     ) -> Result<()> {
+        cfg_if::cfg_if! {
+            if #[cfg(feature = "cbor")] {
+                let import_cbor = "minicbor,";
+            }  else {
+                let import_cbor = "";
+            }
+        }
         w.write(
-            r#"// This file is generated automatically using wasmcloud-weld and smithy model definitions
+            r#"// This file is generated automatically using wasmcloud/weld-codegen and smithy model definitions
                //
             "#);
         match &self.namespace {
@@ -169,7 +183,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                 // the base model has minimal dependencies
                 w.write(
                     r#"
-                #![allow(dead_code)]
+                #![allow(dead_code, clippy::needless_lifetimes)]
                 use serde::{{Deserialize, Serialize}};
              "#,
                 );
@@ -181,17 +195,16 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                 // if the crate we are generating is "wasmbus_rpc" then we have to import it with "crate::".
                 w.write(&format!(
                     r#"
-                #![allow(clippy::ptr_arg)]
-                #[allow(unused_imports)]
+                #![allow(unused_imports, clippy::ptr_arg, clippy::needless_lifetimes)]
                 use {}::{{
                     Context, deserialize, serialize, MessageDispatch, RpcError, RpcResult,
-                    Timestamp, Transport, Message, SendOpts,
+                    Timestamp, Transport, Message, SendOpts, {}
                 }};
-                #[allow(unused_imports)] use serde::{{Deserialize, Serialize}};
-                #[allow(unused_imports)] use async_trait::async_trait;
-                #[allow(unused_imports)] use std::{{borrow::Cow, string::ToString}};
+                use serde::{{Deserialize, Serialize}};
+                use async_trait::async_trait;
+                use std::{{borrow::Cow, io::Write, string::ToString}};
                 "#,
-                    &self.import_core
+                    &self.import_core, import_cbor,
                 ));
             }
         }
@@ -202,12 +215,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         Ok(())
     }
 
-    fn declare_types(
-        &mut self,
-        mut w: &mut Writer,
-        model: &Model,
-        _params: &ParamMap,
-    ) -> Result<()> {
+    fn declare_types(&mut self, w: &mut Writer, model: &Model, _params: &ParamMap) -> Result<()> {
         let ns = self.namespace.clone();
 
         let mut shapes = model
@@ -221,14 +229,14 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         for (id, traits, shape) in shapes.into_iter() {
             match shape {
                 ShapeKind::Simple(simple) => {
-                    self.declare_simple_shape(&mut w, id.shape_name(), traits, simple)?;
+                    self.declare_simple_shape(w, id.shape_name(), traits, simple)?;
                 }
                 ShapeKind::Map(map) => {
-                    self.declare_map_shape(&mut w, id.shape_name(), traits, map)?;
+                    self.declare_map_shape(w, id.shape_name(), traits, map)?;
                 }
                 ShapeKind::List(list) => {
                     self.declare_list_or_set_shape(
-                        &mut w,
+                        w,
                         id.shape_name(),
                         traits,
                         list,
@@ -237,7 +245,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                 }
                 ShapeKind::Set(set) => {
                     self.declare_list_or_set_shape(
-                        &mut w,
+                        w,
                         id.shape_name(),
                         traits,
                         set,
@@ -246,7 +254,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                 }
                 ShapeKind::Structure(strukt) => {
                     //if !traits.contains_key(&prelude_shape_named(TRAIT_TRAIT).unwrap()) {
-                    self.declare_structure_shape(&mut w, id.shape_name(), traits, strukt)?;
+                    self.declare_structure_shape(w, id.shape_name(), traits, strukt)?;
                     //}
                 }
                 ShapeKind::Operation(_)
@@ -255,16 +263,17 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                 | ShapeKind::Union(_)
                 | ShapeKind::Unresolved => {}
             }
+            cfg_if::cfg_if! {
+                if #[cfg(feature = "cbor")] {
+                    self.declare_shape_encoder(w, id, shape)?;
+                    self.declare_shape_decoder(w, id, shape)?;
+                }
+            }
         }
         Ok(())
     }
 
-    fn write_services(
-        &mut self,
-        mut w: &mut Writer,
-        model: &Model,
-        _params: &ParamMap,
-    ) -> Result<()> {
+    fn write_services(&mut self, w: &mut Writer, model: &Model, _params: &ParamMap) -> Result<()> {
         let ns = self.namespace.clone();
         for (id, traits, shape) in model
             .shapes()
@@ -272,9 +281,9 @@ impl<'model> CodeGen for RustCodeGen<'model> {
             .map(|s| (s.id(), s.traits(), s.body()))
         {
             if let ShapeKind::Service(service) = shape {
-                self.write_service_interface(&mut w, model, id.shape_name(), traits, service)?;
-                self.write_service_receiver(&mut w, model, id.shape_name(), traits, service)?;
-                self.write_service_sender(&mut w, model, id.shape_name(), traits, service)?;
+                self.write_service_interface(w, model, id.shape_name(), traits, service)?;
+                self.write_service_receiver(w, model, id.shape_name(), traits, service)?;
+                self.write_service_sender(w, model, id.shape_name(), traits, service)?;
             }
         }
         Ok(())
@@ -309,14 +318,14 @@ impl<'model> RustCodeGen<'model> {
     /// Apply documentation traits: (documentation, deprecated, unstable)
     fn apply_documentation_traits(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         id: &Identifier,
         traits: &AppliedTraits,
     ) {
         if let Some(Some(Value::String(text))) =
             traits.get(&prelude_shape_named(TRAIT_DOCUMENTATION).unwrap())
         {
-            self.write_documentation(&mut w, id, text);
+            self.write_documentation(w, id, text);
         }
 
         // deprecated
@@ -338,21 +347,30 @@ impl<'model> RustCodeGen<'model> {
             .get(&prelude_shape_named(TRAIT_UNSTABLE).unwrap())
             .is_some()
         {
-            self.write_comment(&mut w, CommentKind::Documentation, "@unstable");
+            self.write_comment(w, CommentKind::Documentation, "@unstable");
         }
     }
 
+    /// field type, wrapped with Option if field is not required
+    pub(crate) fn field_type_string(&self, field: &MemberShape) -> Result<String> {
+        self.type_string(if is_optional_type(field) {
+            Ty::Opt(field.target())
+        } else {
+            Ty::Shape(field.target())
+        })
+    }
     /// Write a type name, a primitive or defined type, with or without deref('&') and with or without Option<>
-    fn write_type(&mut self, w: &mut Writer, ty: Ty<'_>) -> Result<()> {
+    pub(crate) fn type_string(&self, ty: Ty<'_>) -> Result<String> {
+        let mut s = String::new();
         match ty {
             Ty::Opt(id) => {
-                w.write(b"Option<");
-                self.write_type(w, Ty::Shape(id))?;
-                w.write(b">");
+                s.push_str("Option<");
+                s.push_str(&self.type_string(Ty::Shape(id))?);
+                s.push('>');
             }
             Ty::Ref(id) => {
-                w.write(b"&");
-                self.write_type(w, Ty::Shape(id))?;
+                s.push('&');
+                s.push_str(&self.type_string(Ty::Shape(id))?);
             }
             Ty::Shape(id) => {
                 let name = id.shape_name().to_string();
@@ -385,59 +403,65 @@ impl<'model> RustCodeGen<'model> {
                         }
                         _ => return Err(Error::UnsupportedType(name)),
                     };
-                    w.write(ty);
+                    s.push_str(ty);
                 } else if id.namespace() == wasmcloud_model_namespace() {
-                    match name.as_bytes() {
-                        b"U64" | b"U32" | b"U16" | b"U8" => {
-                            w.write(b"u");
-                            w.write(&name.as_bytes()[1..])
+                    match name.as_str() {
+                        "U64" | "U32" | "U16" | "U8" => {
+                            s.push('u');
+                            s.push_str(&name[1..])
                         }
-                        b"I64" | b"I32" | b"I16" | b"I8" => {
-                            w.write(b"i");
-                            w.write(&name.as_bytes()[1..])
+                        "I64" | "I32" | "I16" | "I8" => {
+                            s.push('i');
+                            s.push_str(&name[1..]);
                         }
                         _ => {
                             if self.namespace.is_none()
                                 || self.namespace.as_ref().unwrap() != wasmcloud_model_namespace()
                             {
-                                w.write(&self.import_core);
-                                w.write(b"::model::");
+                                s.push_str(&self.import_core);
+                                s.push_str("::model::");
                             }
-                            w.write(&self.to_type_name(&name));
+                            s.push_str(&self.to_type_name(&name));
                         }
                     };
                 } else if self.namespace.is_some()
                     && id.namespace() == self.namespace.as_ref().unwrap()
                 {
                     // we are in the same namespace so we don't need to specify namespace
-                    w.write(&self.to_type_name(&id.shape_name().to_string()));
+                    s.push_str(&self.to_type_name(&id.shape_name().to_string()));
                 } else {
                     match self.packages.get(&id.namespace().to_string()) {
                         Some(package) => {
                             // the crate name should be valid rust syntax. If not, they'll get an error with rustc
-                            w.write(&package.crate_name);
-                            w.write(b"::");
-                            w.write(&self.to_type_name(&id.shape_name().to_string()));
+                            s.push_str(&package.crate_name);
+                            s.push_str("::");
+                            s.push_str(&self.to_type_name(&id.shape_name().to_string()));
                         }
                         None => {
                             return Err(Error::Model(format!("undefined create for namespace {} for symbol {}. Make sure codegen.toml includes all dependent namespaces",
-                                    &id.namespace(), &id)));
+                                                            &id.namespace(), &id)));
                         }
                     }
                 }
             }
         }
+        Ok(s)
+    }
+
+    /// Write a type name, a primitive or defined type, with or without deref('&') and with or without Option<>
+    fn write_type(&mut self, w: &mut Writer, ty: Ty<'_>) -> Result<()> {
+        w.write(&self.type_string(ty)?);
         Ok(())
     }
 
     /// append suffix to type name, for example "Game", "Context" -> "GameContext"
     fn write_ident_with_suffix(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         id: &Identifier,
         suffix: &str,
     ) -> Result<()> {
-        self.write_ident(&mut w, id);
+        self.write_ident(w, id);
         w.write(suffix); // assume it's already PascalCalse
         Ok(())
     }
@@ -445,14 +469,14 @@ impl<'model> RustCodeGen<'model> {
     // declaration for simple type
     fn declare_simple_shape(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         id: &Identifier,
         traits: &AppliedTraits,
         simple: &Simple,
     ) -> Result<()> {
-        self.apply_documentation_traits(&mut w, id, traits);
+        self.apply_documentation_traits(w, id, traits);
         w.write(b"pub type ");
-        self.write_ident(&mut w, id);
+        self.write_ident(w, id);
         w.write(b" = ");
         let ty = match simple {
             Simple::Blob => "Vec<u8>",
@@ -486,52 +510,52 @@ impl<'model> RustCodeGen<'model> {
 
     fn declare_map_shape(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         id: &Identifier,
         traits: &AppliedTraits,
         shape: &MapShape,
     ) -> Result<()> {
-        self.apply_documentation_traits(&mut w, id, traits);
+        self.apply_documentation_traits(w, id, traits);
         w.write(b"pub type ");
-        self.write_ident(&mut w, id);
+        self.write_ident(w, id);
         w.write(b" = ");
         w.write(DEFAULT_MAP_TYPE);
         w.write(b"<");
-        self.write_type(&mut w, Ty::Shape(shape.key().target()))?;
+        self.write_type(w, Ty::Shape(shape.key().target()))?;
         w.write(b",");
-        self.write_type(&mut w, Ty::Shape(shape.value().target()))?;
+        self.write_type(w, Ty::Shape(shape.value().target()))?;
         w.write(b">;\n\n");
         Ok(())
     }
 
     fn declare_list_or_set_shape(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         id: &Identifier,
         traits: &AppliedTraits,
         shape: &ListOrSet,
         typ: &str,
     ) -> Result<()> {
-        self.apply_documentation_traits(&mut w, id, traits);
+        self.apply_documentation_traits(w, id, traits);
         w.write(b"pub type ");
-        self.write_ident(&mut w, id);
+        self.write_ident(w, id);
         w.write(b" = ");
         w.write(typ);
         w.write(b"<");
-        self.write_type(&mut w, Ty::Shape(shape.member().target()))?;
+        self.write_type(w, Ty::Shape(shape.member().target()))?;
         w.write(b">;\n\n");
         Ok(())
     }
 
     fn declare_structure_shape(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         id: &Identifier,
         traits: &AppliedTraits,
         strukt: &StructureOrUnion,
     ) -> Result<()> {
         let is_trait_struct = traits.contains_key(&prelude_shape_named(TRAIT_TRAIT).unwrap());
-        self.apply_documentation_traits(&mut w, id, traits);
+        self.apply_documentation_traits(w, id, traits);
         let mut derive_list = vec!["Clone", "Debug", "PartialEq", "Serialize", "Deserialize"];
         // derive(Default) is disabled for traits and enabled for all other structs
         let mut derive_default = !is_trait_struct;
@@ -551,7 +575,7 @@ impl<'model> RustCodeGen<'model> {
         let derive_decl = format!("#[derive({})]\n", derive_list.join(","));
         w.write(&derive_decl);
         w.write(b"pub struct ");
-        self.write_ident(&mut w, id);
+        self.write_ident(w, id);
         w.write(b" {\n");
         // sort fields for deterministic output
         let mut fields = strukt
@@ -560,7 +584,7 @@ impl<'model> RustCodeGen<'model> {
             .collect::<Vec<MemberShape>>();
         fields.sort_by_key(|f| f.id().to_owned());
         for member in fields.iter() {
-            self.apply_documentation_traits(&mut w, member.id(), member.traits());
+            self.apply_documentation_traits(w, member.id(), member.traits());
 
             // use the declared name for serialization, unless an override is declared
             // with `@sesrialization(name: SNAME)`
@@ -625,46 +649,37 @@ impl<'model> RustCodeGen<'model> {
                 // See the comment for has_default for more info.
                 w.write(r#"  #[serde(default)] "#);
             }
-            w.write(b"  pub ");
-            w.write(&rust_field_name);
-            w.write(b": ");
-            self.write_field_type(&mut w, member)?;
-            w.write(b",\n");
+            w.write(
+                format!(
+                    "  pub {}: {},\n",
+                    &rust_field_name,
+                    self.field_type_string(member)?
+                )
+                .as_bytes(),
+            );
         }
         w.write(b"}\n\n");
         Ok(())
     }
 
-    /// write field type, wrapping with Option if field is not required
-    fn write_field_type(&mut self, mut w: &mut Writer, field: &MemberShape) -> Result<()> {
-        self.write_type(
-            &mut w,
-            if is_optional_type(field) {
-                Ty::Opt(field.target())
-            } else {
-                Ty::Shape(field.target())
-            },
-        )
-    }
-
     /// Declares the service as a rust Trait whose methods are the smithy service operations
     fn write_service_interface(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         model: &Model,
         service_id: &Identifier,
         service_traits: &AppliedTraits,
         service: &Service,
     ) -> Result<()> {
-        self.apply_documentation_traits(&mut w, service_id, service_traits);
+        self.apply_documentation_traits(w, service_id, service_traits);
 
         #[cfg(feature = "wasmbus")]
-        self.add_wasmbus_comments(&mut w, service_id, service_traits)?;
+        self.add_wasmbus_comments(w, service_id, service_traits)?;
 
         w.write(b"#[async_trait]\npub trait ");
-        self.write_ident(&mut w, service_id);
+        self.write_ident(w, service_id);
         w.write(b"{\n");
-        self.write_service_contract_getter(&mut w, service_id, service_traits)?;
+        self.write_service_contract_getter(w, service_id, service_traits)?;
 
         for operation in service.operations() {
             // if operation is not declared in this namespace, don't define it here
@@ -677,7 +692,7 @@ impl<'model> RustCodeGen<'model> {
 
             // TODO: re-think what to do if operation is in another namespace and self.namespace is None
             let method_id = operation.shape_name();
-            let _flags = self.write_method_signature(&mut w, method_id, op_traits, op)?;
+            let _flags = self.write_method_signature(w, method_id, op_traits, op)?;
             w.write(b";\n");
         }
         w.write(b"}\n\n");
@@ -710,7 +725,7 @@ impl<'model> RustCodeGen<'model> {
     #[cfg(feature = "wasmbus")]
     fn add_wasmbus_comments(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         service_id: &Identifier,
         service_traits: &AppliedTraits,
     ) -> Result<()> {
@@ -719,15 +734,15 @@ impl<'model> RustCodeGen<'model> {
         if let Some(wasmbus) = wasmbus {
             if let Some(contract) = wasmbus.contract_id {
                 let text = format!("wasmbus.contractId: {}", &contract);
-                self.write_documentation(&mut w, service_id, &text);
+                self.write_documentation(w, service_id, &text);
             }
             if wasmbus.provider_receive {
                 let text = "wasmbus.providerReceive";
-                self.write_documentation(&mut w, service_id, text);
+                self.write_documentation(w, service_id, text);
             }
             if wasmbus.actor_receive {
                 let text = "wasmbus.actorReceive";
-                self.write_documentation(&mut w, service_id, text);
+                self.write_documentation(w, service_id, text);
             }
         }
         Ok(())
@@ -737,14 +752,14 @@ impl<'model> RustCodeGen<'model> {
     /// does not write trailing semicolon so this can be used for declaration and implementation
     fn write_method_signature(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         method_id: &Identifier,
         method_traits: &AppliedTraits,
         op: &Operation,
     ) -> Result<MethodArgFlags> {
         let method_name = self.to_method_name(method_id);
         let mut arg_flags = MethodArgFlags::Normal;
-        self.apply_documentation_traits(&mut w, method_id, method_traits);
+        self.apply_documentation_traits(w, method_id, method_traits);
         w.write(b"async fn ");
         w.write(&method_name);
         if let Some(input_type) = op.input() {
@@ -759,12 +774,12 @@ impl<'model> RustCodeGen<'model> {
             if matches!(arg_flags, MethodArgFlags::ToString) {
                 w.write(b"&TS");
             } else {
-                self.write_type(&mut w, Ty::Ref(input_type))?;
+                self.write_type(w, Ty::Ref(input_type))?;
             }
         }
         w.write(b") -> RpcResult<");
         if let Some(output_type) = op.output() {
-            self.write_type(&mut w, Ty::Shape(output_type))?;
+            self.write_type(w, Ty::Shape(output_type))?;
         } else {
             w.write(b"()");
         }
@@ -775,7 +790,7 @@ impl<'model> RustCodeGen<'model> {
     // pub trait FooReceiver : MessageDispatch + Foo { ... }
     fn write_service_receiver(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         model: &Model,
         service_id: &Identifier,
         service_traits: &AppliedTraits,
@@ -785,12 +800,12 @@ impl<'model> RustCodeGen<'model> {
             "{}Receiver receives messages defined in the {} service trait",
             service_id, service_id
         );
-        self.write_comment(&mut w, CommentKind::Documentation, &doc);
-        self.apply_documentation_traits(&mut w, service_id, service_traits);
+        self.write_comment(w, CommentKind::Documentation, &doc);
+        self.apply_documentation_traits(w, service_id, service_traits);
         w.write(b"#[doc(hidden)]\n#[async_trait]\npub trait ");
-        self.write_ident_with_suffix(&mut w, service_id, "Receiver")?;
+        self.write_ident_with_suffix(w, service_id, "Receiver")?;
         w.write(b" : MessageDispatch + ");
-        self.write_ident(&mut w, service_id);
+        self.write_ident(w, service_id);
         w.write(
             br#"{
             async fn dispatch(
@@ -818,13 +833,13 @@ impl<'model> RustCodeGen<'model> {
                 // let value : InputType = deserialize(...)?;
                 w.write(b"let value: ");
                 // TODO: should this be input.target?
-                self.write_type(&mut w, Ty::Shape(op.input().as_ref().unwrap()))?;
+                self.write_type(w, Ty::Shape(op.input().as_ref().unwrap()))?;
                 w.write(b" = deserialize(message.arg.as_ref())\
                   .map_err(|e| RpcError::Deser(format!(\"message '{}': {}\", message.method, e)))?;\n");
             }
             // let resp = Trait::method(self, ctx, &value).await?;
             w.write(b"let resp = ");
-            self.write_ident(&mut w, service_id); // Service::method
+            self.write_ident(w, service_id); // Service::method
             w.write(b"::");
             w.write(&self.to_method_name(method_ident));
             w.write(b"(self, ctx");
@@ -841,7 +856,7 @@ impl<'model> RustCodeGen<'model> {
             w.write(b"\", arg: buf })},\n");
         }
         w.write(b"_ => Err(RpcError::MethodNotHandled(format!(\"");
-        self.write_ident(&mut w, service_id);
+        self.write_ident(w, service_id);
         w.write(b"::{}\", message.method))),\n");
         w.write(b"}\n}\n}\n\n"); // end match, end fn dispatch, end trait
 
@@ -852,7 +867,7 @@ impl<'model> RustCodeGen<'model> {
     // pub struct FooSender{ ... }
     fn write_service_sender(
         &mut self,
-        mut w: &mut Writer,
+        w: &mut Writer,
         model: &Model,
         service_id: &Identifier,
         service_traits: &AppliedTraits,
@@ -862,8 +877,8 @@ impl<'model> RustCodeGen<'model> {
             "{}Sender sends messages to a {} service",
             service_id, service_id
         );
-        self.write_comment(&mut w, CommentKind::Documentation, &doc);
-        self.apply_documentation_traits(&mut w, service_id, service_traits);
+        self.write_comment(w, CommentKind::Documentation, &doc);
+        self.apply_documentation_traits(w, service_id, service_traits);
         w.write(&format!(
             r#"/// client for sending {} messages
               #[derive(Debug)]
@@ -874,6 +889,10 @@ impl<'model> RustCodeGen<'model> {
                   pub fn via(transport: T) -> Self {{
                       Self{{ transport }}
                   }}
+                  
+                  pub fn set_timeout(&self, interval: std::time::Duration) {{
+                    self.transport.set_timeout(interval);
+                  }}
               }}
             "#,
             service_id, service_id, service_id, service_id,
@@ -885,9 +904,9 @@ impl<'model> RustCodeGen<'model> {
 
         // implement Trait for TraitSender
         w.write(b"#[async_trait]\nimpl<T:Transport + std::marker::Sync + std::marker::Send> ");
-        self.write_ident(&mut w, service_id);
+        self.write_ident(w, service_id);
         w.write(b" for ");
-        self.write_ident_with_suffix(&mut w, service_id, "Sender")?;
+        self.write_ident_with_suffix(w, service_id, "Sender")?;
         w.write(b"<T> {\n");
 
         for method_id in service.operations() {
@@ -902,7 +921,7 @@ impl<'model> RustCodeGen<'model> {
 
             let (op, method_traits) = get_operation(model, method_id, service_id)?;
             w.write(b"#[allow(unused)]\n");
-            let arg_flags = self.write_method_signature(&mut w, method_ident, method_traits, op)?;
+            let arg_flags = self.write_method_signature(w, method_ident, method_traits, op)?;
             w.write(b" {\n");
             if op.has_input() {
                 if matches!(arg_flags, MethodArgFlags::ToString) {
@@ -1045,7 +1064,7 @@ impl<'model> RustCodeGen<'model> {
 /// is_optional_type determines whether the field should be wrapped in Option<>
 /// the value is true if it has an explicit `box` trait, or if it's
 /// un-annotated and not one of (boolean, byte, short, integer, long, float, double)
-fn is_optional_type(field: &MemberShape) -> bool {
+pub(crate) fn is_optional_type(field: &MemberShape) -> bool {
     field.is_boxed()
         || (!field.is_required()
             && ![
