@@ -187,7 +187,7 @@ pub fn trait_value<T: DeserializeOwned>(value: &NodeValue) -> Result<T> {
 pub fn value_to_json(value: &NodeValue) -> JsonValue {
     match value {
         NodeValue::None => JsonValue::Null,
-        NodeValue::Array(v) => JsonValue::Array(v.iter().map(|v| value_to_json(v)).collect()),
+        NodeValue::Array(v) => JsonValue::Array(v.iter().map(value_to_json).collect()),
         NodeValue::Object(v) => {
             let mut object = crate::JsonMap::default();
             for (k, v) in v {
@@ -234,7 +234,6 @@ pub fn has_default(model: &'_ Model, member: &MemberShape) -> bool {
 
     if id.namespace().eq(prelude_namespace_id()) {
         let name = id.shape_name().to_string();
-        eprintln!("has_default name={}", &name);
         cfg_if::cfg_if! {
             if #[cfg(feature = "BigInteger")] {
                 has = has || &name == "bigInteger";
