@@ -57,7 +57,7 @@ const BASIC_TYPES: &[&str] = &[
 ///
 pub type Template<'template> = (&'template str, &'template str);
 
-#[derive(Debug)]
+#[derive(Default, Debug)]
 pub struct RenderConfig<'render> {
     /// Templates to be loaded for renderer. List of template name, data
     pub templates: Vec<Template<'render>>,
@@ -67,15 +67,6 @@ pub struct RenderConfig<'render> {
     pub strict_mode: bool,
 }
 
-impl<'render> Default for RenderConfig<'render> {
-    fn default() -> Self {
-        Self {
-            templates: Vec::default(),
-            strict_mode: false,
-        }
-    }
-}
-
 /// HBTemplate processor for code generation
 pub struct Renderer<'gen> {
     /// Handlebars processor
